/*
 * 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007-2008 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 *
 */
package com.sun.grizzly;

/**
 * An interface used as a wrapper around any kind of thread pool.
 *
 * @param E 
 * @author Jean-Francois Arcand
 */
public interface Pipeline<E> {
    
    
    /**
     * Add an <code>E</code> to be processed by this {@link Pipeline}
     * @param task 
     * @throws com.sun.grizzly.PipelineFullException 
     */
    public void execute(E task) throws PipelineFullException;


    /**
     * Return a <code>E</code> object available in the pipeline.
     * @return
     */
    public E waitForIoTask() ;
    
   
   /**
     * Return the number of waiting threads.
     * @return number of waiting threads
    */
    public int getWaitingThread();
    
    
    /** 
     * Return the number of threads used by this pipeline.
     * @return max number of threads
     */
    public int getMaxThreads();
    
    
    /**
     * Return the number of active threads.
     * @return number of active threads
     */
    public int getCurrentThreadCount() ;
      
      
    /**
     * Return the curent number of threads that are currently processing 
     * a task.
     * @return number of currently processing threads
     */
    public int getCurrentThreadsBusy();
    
   
    /**
     * Init the {@link Pipeline} by initializing the required
     * {@link WorkerThread}. Default value is 10
     */
    public void initPipeline();


    /**
     * Return the name of this {@link Pipeline}
     * @return name of this {@link Pipeline}
     */
    public String getName();


    /**
     * Start the {@link Pipeline}
     */
    public void startPipeline();
    

    /**
     * Stop the {@link Pipeline} 
     */
    public void stopPipeline();

    
    /**
     * Set the {@link Thread} priority used when creating new threads.
     * @param priority 
     */
    public void setPriority(int priority);
    
    
    /**
     * Set the maximum thread this pipeline can handle.
     * @param maxThread 
     */
    public void setMaxThreads(int maxThread);
    
    
    /**
     * Set the minimum thread this pipeline can handle.
     * @param minThread 
     */    
    public void setMinThreads(int minThread);
    
    
    /**
     * Set the name of this {@link Pipeline}
     * @param name 
     */
    public void setName(String name);
   
    
    /**
     * Set the maximum pending connection this {@link Pipeline}
     * can handle.
     * @param maxQueueSizeInBytesCount 
     * @depre
     */
    public void setMaxQueueSize(int maxQueue);
    
   
    /**
     * Set the number the {@link Pipeline} will use when increasing the 
     * thread pool
     * @param processorThreadsIncrement 
     */
    public void setThreadsIncrement(int threadIncrements);
    
    
    /**
     * Returns the number of tasks in this {@link Pipeline}.
     *
     * @return Number of tasks in this {@link Pipeline}.
     */
    public int size();
}
