/*
 * 
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 2007-2008 Sun Microsystems, Inc. All rights reserved.
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License. You can obtain
 * a copy of the License at https://glassfish.dev.java.net/public/CDDL+GPL.html
 * or glassfish/bootstrap/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 * 
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 * Sun designates this particular file as subject to the "Classpath" exception
 * as provided by Sun in the GPL Version 2 section of the License file that
 * accompanied this code.  If applicable, add the following below the License
 * Header, with the fields enclosed by brackets [] replaced by your own
 * identifying information: "Portions Copyrighted [year]
 * [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 *
 */

package com.sun.grizzly.tcp.http11;

import java.util.ArrayList;

/**
 * The GrizzlyAdapterChain class allow the invokation of multiple {@link GrizzlyAdapter}
 * every time a nre HTTP requests is reasdy to be handled. {@link GrizzlyAdapter}
 * are called one by one, and the invokation stops when a {@link GrizzlyAdapter}
 * set the {@link GrizzlyResponse#getStatus} return the Http code <tt>200</tt>, 
 * which will stop the chain invokation. 
 * 
 * Note: This class is <strong>NOT</strong> thread-safe, so make sure you synchronize
 *       when dynamically adding and removing {@link GrizzlyAdapter}
 *  
 * @author Jeanfrancois Arcand
 */
public class GrizzlyAdapterChain extends GrizzlyAdapter{

    /**
     * The list of {@link GrizzlyAdapter} instance.
     */
    private ArrayList<GrizzlyAdapter> adapters = 
            new ArrayList<GrizzlyAdapter>();
    
    
    
    /**
     {@link GrizzlyAdapter} are called one by one, and the invokation stops when
     * set the {@link GrizzlyResponse#getStatus} return the Http code <tt>200</tt>, 
     * which will stop the chain invokation. 
     * @param request The {@link GrizzlyRequest}
     * @param response The {@link GrizzlyResponse}
     */
    @Override
    public void service(GrizzlyRequest request, GrizzlyResponse response) {    
        int size =  adapters.size();
        for (int i=0; i < size; i++){          
            adapters.get(i).service(request, response);
            if (response.getStatus() == 404 && i != size -1){
                response.setStatus(200, "OK");
            } else {
                return;
            }
        }  
    }
    
    
    /**
     * Add a {@link GrizzlyAdapter} to the chain.
     * @param 
     */
    public void addGrizzlyAdapter(GrizzlyAdapter adapter){
        adapters.add(adapter);
    }
    
    
    /**
     * Remove a {@link GrizzlyAdapter}
     * @return <tt>true</tt> if removed
     */
    public boolean removeAdapter(GrizzlyAdapter adapter){
        return adapters.remove(adapter);
    }
}
